@@ -3,6 +3,7 @@ from __future__ import annotations
 import io
 import logging
 import tarfile
+import zipfile
 from typing import TYPE_CHECKING, Any, TypedDict
 
 from pydantic import BaseModel
@@ -16,29 +17,59 @@ if TYPE_CHECKING:
 
 logger = logging.getLogger("hud.utils.common")
 
-class HudStyleConfig(BaseModel):
-    function: str  # Format: "x.y.z"
-    args: list[Any] # Must be json serializable
 
-    id: str | None = None # Optional id for remote execution
+class FunctionConfig(BaseModel):
+    function: str  # Format: "x.y.z"
+    args: list[Any]  # Must be json serializable
+
+    id: str | None = None  # Optional id for remote execution
+    metadata: dict[str, Any] | None = None  # Optional metadata for telemetry
 
     def __len__(self) -> int:
         return len(self.args)
 
     def __getitem__(self, index: int) -> Any:
         return self.args[index]
-    
+
     def __iter__(self) -> Iterator[Any]:
         return iter(self.args)
-    
+
     def __str__(self) -> str:
-        return f"{self.function}: {', '.join(str(arg) for arg in self.args)}"
+        return f"FC: {self.function}: {', '.join(str(arg) for arg in self.args)} ({self.metadata})"
+
 
 # Type alias for the shorthand config, which just converts to function name and args
-ShorthandConfig = tuple[str | dict[str, Any] | list[str] | list[dict[str, Any]], ...]
+BasicType = str | int | float | bool | None
+ShorthandConfig = tuple[BasicType | dict[str, Any] | list[BasicType] | list[dict[str, Any]], ...]
 
 # Type alias for multiple config formats
-HudStyleConfigs = ShorthandConfig | HudStyleConfig | list[HudStyleConfig] | dict[str, Any] | str
+FunctionConfigs = (
+    ShorthandConfig
+    | FunctionConfig
+    | list[FunctionConfig]
+    | list[ShorthandConfig]
+    | dict[str, Any]
+    | str
+)
+
+
+class Observation(BaseModel):
+    """
+    Observation from the environment.
+
+    Attributes:
+        screenshot: Base64 encoded PNG string of the screen
+        text: Text observation, if available
+    """
+
+    screenshot: str | None = None  # base64 string png
+    text: str | None = None
+
+    def __str__(self) -> str:
+        return f"""Observation(screenshot={
+            self.screenshot[:100] if self.screenshot else "None"
+        }..., text={self.text}...)"""
+
 
 class ExecuteResult(TypedDict):
     """
@@ -49,26 +80,27 @@ class ExecuteResult(TypedDict):
         stderr: Standard error from the command
         exit_code: Exit code of the command
     """
+
     stdout: bytes
     stderr: bytes
     exit_code: int
-    
-    
+
+
 def directory_to_tar_bytes(directory_path: Path) -> bytes:
     """
     Converts a directory to a tar archive and returns it as bytes.
-    
+
     This function creates a tar archive of the specified directory in memory,
     without writing to a temporary file on disk.
-    
+
     Args:
         path: Path to the directory to convert
-        
+
     Returns:
         Bytes of the tar archive
     """
     output = io.BytesIO()
-    
+
     with tarfile.open(fileobj=output, mode="w") as tar:
         # Walk through the directory
         for file_path in directory_path.rglob("*"):
@@ -77,12 +109,24 @@ def directory_to_tar_bytes(directory_path: Path) -> bytes:
                 rel_path = file_path.relative_to(directory_path)
                 logger.debug("Adding %s to tar archive", rel_path)
                 tar.add(file_path, arcname=str(rel_path))
-    
+
     # Get the bytes from the BytesIO object
     output.seek(0)
     return output.getvalue()
 
 
+def directory_to_zip_bytes(context_dir: Path) -> bytes:
+    """Zip a directory and return the zip archive as bytes."""
+    output = io.BytesIO()
+    with zipfile.ZipFile(output, "w", zipfile.ZIP_DEFLATED) as zipf:
+        for file_path in context_dir.rglob("*"):
+            if file_path.is_file():
+                rel_path = file_path.relative_to(context_dir)
+                logger.debug("Adding %s to zip archive", rel_path)
+                zipf.write(str(file_path), arcname=str(rel_path))
+    return output.getvalue()
+
+
 async def get_gym_id(gym_name_or_id: str) -> str:
     """
     Get the gym ID for a given gym name or ID.
